// ============================================================================
//                  a m d b a   m e s h   u t i l i t i e s
// ============================================================================
//
// nb_nodes nb_elts type_elt (SEGMENT, TRIANGLE, QUADRANGLE, TETRAEDRE, HEXAEDRE)
// n1 x1 (y1) (z1) iref1
// n2 x2 (y2) (z2) iref2
// ....
// e1 i1 j1 k1 ... iref1
// e2 i2 j2 k2 ... iref2
// ...
//
// WARNING : only same type of element of order 1
// ============================================================================

#include "mesh.h"

extern map<uint,element_type> element_type_list;
extern map<string,mesh_type> mesh_type_list;


// amdba mesh loading
void mesh::loadfrom_amdba(const char * filename) {
    push_context("mesh::loadfrom_amdba");
    message_event("err_filein2",msg_data("amdba"));
    pop_context();
}

// save mesh to amdba format file
void mesh::saveto_amdba(const char * filename) {
    push_context("mesh::saveto_amdba");

    // first order element compatibility
    string error;
    uint t=0;

    for (cit_le it=elements_list.begin(); it!=elements_list.end(); ++it) {
        if (element_type_list[it->second.type].dim==dim) { // restriction to domain element
            if (element_type_list[it->second.type].order>1) {
                message_event("err_filein3",msg_data("amdba"));
            }

            if (t==0) {
                t=it->second.type;

            } else if (t!=it->second.type) {
                message_event("err_filein4",msg_data("amdba"));
            }
        }
    }
    
    // open file
    string savename;

    if (filename!=NULL) {
        savename=string(filename);
    }

    if (!valid_savefile(savename,string("ambda"))) {
        return;
    }

    ofstream f(savename.c_str(),ios::out | ios::binary);

    if (f.fail()) {
        message_event("open_fail",msg_data(savename));
    }
    
    // print header
    uint nb_elts=0;

    for (cit_le it=elements_list.begin(); it!=elements_list.end(); ++it)
        if (element_type_list[it->second.type].dim==dim) {
            nb_elts++;
        }

    f << nodes_list.size() << " " << nb_elts << " " << element_type_list[t].amdba_name << endl;
    
    // construct node ref from domain definition (inconsistent if a node belongs to two domains at least!)
    for (cit_le it=elements_list.begin(); it!=elements_list.end(); ++it) {
        if (element_type_list[it->second.type].dim<dim) { // restriction to boundary element
            const element &E=it->second;
            uint n=E.nodes.size();

            for (uint i=0; i<n; i++) {
                if (nodes_list(E.nodes[i]).ref!=0) {
                    message_event("warn_amdba",msg_data(int(E.nodes[i])));
                }

                nodes_list(E.nodes[i]).ref=E.num_domain;
            }
        }
    }
    
    // print nodes
    for (cit_ln it=nodes_list.begin(); it!=nodes_list.end(); ++it) {
        f << it->first;

        for (uint i=0; i<dim; i++) {
            f << " " << it->second.x[i];
        }

        f << " " << it->second.ref << endl;
    }
    
    // print elements
    uint l=0;

    for (cit_le it=elements_list.begin(); it!=elements_list.end(); ++it) {
        if (element_type_list[it->second.type].dim==dim) {
            l++;
            f << l;

            for (uint i=0; i<it->second.nodes.size(); i++) {
                f << " " << it->second.nodes[i];
            }

            f << " " << it->second.num_domain << endl;
        }
    }
    
    // close file
    f.close();
    pop_context();
    // end of saving mesh to amdba file
}
